@@ -46,12 +46,15 @@ from tensorflow_datasets.core.splits import SubSplitInfo
 from tensorflow_datasets.core.tfrecords_reader import ReadInstruction
 
 from tensorflow_datasets.core.utils import Experiment
+from tensorflow_datasets.core.utils import gcs_path
 from tensorflow_datasets.core.utils import get_tfds_path
 from tensorflow_datasets.core.utils import Version
+from tensorflow_datasets.core.utils.benchmark import benchmark
 
 
 __all__ = [
     "add_data_dir",
+    "benchmark",
     "BeamBasedBuilder",
     "BeamMetadataDict",
     "BuilderConfig",
@@ -61,6 +64,7 @@ __all__ = [
     "Experiment",
     "GeneratorBasedBuilder",
     "get_tfds_path",
+    "gcs_path",
     "lazy_imports",
     "Metadata",
     "MetadataDict",
