@@ -27,11 +27,15 @@ import numpy as np
 import six
 import tensorflow.compat.v2 as tf
 from tensorflow_datasets import testing
+from tensorflow_datasets.core import dataset_builder
 from tensorflow_datasets.core import dataset_info
 from tensorflow_datasets.core import features
 from tensorflow_datasets.core.utils import py_utils
 from tensorflow_datasets.image_classification import mnist
 
+from google.protobuf import text_format
+from tensorflow_metadata.proto.v0 import schema_pb2
+
 tf.enable_v2_behavior()
 
 _TFDS_DIR = py_utils.tfds_dir()
@@ -73,12 +77,14 @@ class DatasetInfoTest(testing.TestCase):
   @classmethod
   def setUpClass(cls):
     super(DatasetInfoTest, cls).setUpClass()
+    dataset_builder._is_py2_download_and_prepare_disabled = False
     cls._tfds_tmp_dir = testing.make_tmp_dir()
     cls._builder = DummyDatasetSharedGenerator(data_dir=cls._tfds_tmp_dir)
 
   @classmethod
   def tearDownClass(cls):
     super(DatasetInfoTest, cls).tearDownClass()
+    dataset_builder._is_py2_download_and_prepare_disabled = True
     testing.rm_tmp_dir(cls._tfds_tmp_dir)
 
   def test_undefined_dir(self):
@@ -266,22 +272,69 @@ class DatasetInfoTest(testing.TestCase):
       # Per split.
       test_split = builder.info.splits["test"].get_proto()
       train_split = builder.info.splits["train"].get_proto()
-      self.assertEqual(10, test_split.statistics.num_examples)
-      self.assertEqual(20, train_split.statistics.num_examples)
+      expected_schema = text_format.Parse("""
+      feature {
+        name: "x"
+        type: INT
+        presence {
+          min_fraction: 1.0
+          min_count: 1
+        }
+        shape {
+          dim {
+            size: 1
+          }
+        }
+      }""", schema_pb2.Schema())
+      self.assertEqual(train_split.statistics.num_examples, 20)
+      self.assertLen(train_split.statistics.features, 1)
+      self.assertEqual(
+          train_split.statistics.features[0].path.step[0], "x")
+      self.assertLen(
+          train_split.statistics.features[0].num_stats.common_stats.
+          num_values_histogram.buckets, 10)
+      self.assertLen(
+          train_split.statistics.features[0].num_stats.histograms, 2)
+
+      self.assertEqual(test_split.statistics.num_examples, 10)
+      self.assertLen(test_split.statistics.features, 1)
+      self.assertEqual(
+          test_split.statistics.features[0].path.step[0], "x")
+      self.assertLen(
+          test_split.statistics.features[0].num_stats.common_stats.
+          num_values_histogram.buckets, 10)
+      self.assertLen(
+          test_split.statistics.features[0].num_stats.histograms, 2)
+      self.assertEqual(builder.info.as_proto.schema, expected_schema)
 
   @testing.run_in_graph_and_eager_modes()
-  def test_statistics_generation_variable_sizes(self):
+  def test_schema_generation_variable_sizes(self):
     with testing.tmp_dir(self.get_temp_dir()) as tmp_dir:
       builder = RandomShapedImageGenerator(data_dir=tmp_dir)
       builder.download_and_prepare()
 
-      # Get the expected type of the feature.
-      schema_feature = builder.info.as_proto.schema.feature[0]
-      self.assertEqual("im", schema_feature.name)
-
-      self.assertEqual(-1, schema_feature.shape.dim[0].size)
-      self.assertEqual(-1, schema_feature.shape.dim[1].size)
-      self.assertEqual(3, schema_feature.shape.dim[2].size)
+      expected_schema = text_format.Parse(
+          """
+feature {
+  name: "im"
+  type: BYTES
+  presence {
+    min_fraction: 1.0
+    min_count: 1
+  }
+  shape {
+    dim {
+      size: -1
+    }
+    dim {
+      size: -1
+    }
+    dim {
+      size: 3
+    }
+  }
+}""", schema_pb2.Schema())
+      self.assertEqual(builder.info.as_proto.schema, expected_schema)
 
   def test_metadata(self):
     with testing.tmp_dir(self.get_temp_dir()) as tmp_dir:
