@@ -299,11 +299,13 @@ def load(name,
       (for builders with configs, it would be `"foo_bar/zoo/a=True,b=3"` to
       use the `"zoo"` config and pass to the builder keyword arguments `a=True`
       and `b=3`).
-    split: `tfds.Split` or `str`, which split of the data to load. If None,
-      will return a `dict` with all splits (typically `tfds.Split.TRAIN` and
-      `tfds.Split.TEST`).
-    data_dir: `str` (optional), directory to read/write data.
-      Defaults to "~/tensorflow_datasets".
+    split: Which split of the data to load (e.g. `'train'`, `'test'`
+      `['train', 'test']`, `'train[80%:]'`,...). See our
+      [split API guide](https://www.tensorflow.org/datasets/splits).
+      If `None`, will return all splits in a `Dict[Split, tf.data.Dataset]`
+    data_dir: `str`, directory to read/write data. Defaults to the value of
+      the environment variable TFDS_DATA_DIR, if set, otherwise falls back to
+      "~/tensorflow_datasets".
     batch_size: `int`, if set, add a batch dimension to examples. Note that
       variable length features will be 0-padded. If
       `batch_size=-1`, will return the full dataset as `tf.Tensor`s.
@@ -359,7 +361,7 @@ def load(name,
 
   # Set data_dir
   if try_gcs and gcs_utils.is_dataset_on_gcs(name):
-    data_dir = constants.GCS_DATA_DIR
+    data_dir = gcs_utils.gcs_path("datasets")
   elif data_dir is None:
     data_dir = constants.DATA_DIR
 
