@@ -36,20 +36,37 @@ from tensorflow_datasets.core import splits
 from tensorflow_datasets.core import tfrecords_reader
 from tensorflow_datasets.core import tfrecords_writer
 from tensorflow_datasets.core.utils import read_config as read_config_lib
+from tensorflow_datasets.core.utils import shard_utils
 
 
 # Skip the cardinality test for backward compatibility with TF <= 2.1.
 _SKIP_CARDINALITY_TEST = not hasattr(tf.data.experimental, 'assert_cardinality')
 
 
+def _write_tfrecord_from_shard_spec(shard_spec, get):
+  """Write tfrecord shard given shard_spec and buckets to read data from.
+
+  Args:
+    shard_spec: _ShardSpec, the spec for shard to write.
+    get: callable taking the shard index (of bucket) and returning iterator over
+      its elements.
+  """
+  iterators = []
+  for instruction in shard_spec.file_instructions:
+    iterator = get(int(instruction.filename))
+    skip, take = instruction.skip, instruction.take
+    stop = skip+take if take > 0 else None
+    iterators.append(itertools.islice(iterator, skip, stop))
+  tfrecords_writer._write_tfrecord(shard_spec.path, itertools.chain(*iterators))
+
+
 class GetDatasetFilesTest(testing.TestCase):
 
   NAME2SHARD_LENGTHS = {
       'train': [3, 2, 3, 2, 3],  # 13 examples.
   }
 
-  PATH_PATTERN = os.path.normpath(
-      '/foo/bar/mnist-train.tfrecord-0000%d-of-00005')
+  PATH_PATTERN = 'mnist-train.tfrecord-0000%d-of-00005'
 
   def _get_files(self, instruction):
     file_instructions = tfrecords_reader._make_file_instructions_from_absolutes(
@@ -57,27 +74,30 @@ class GetDatasetFilesTest(testing.TestCase):
         name2shard_lengths=self.NAME2SHARD_LENGTHS,
         absolute_instructions=[instruction],
     )
-    for fi in file_instructions.file_instructions:
-      fi['filename'] = os.path.normpath(
-          os.path.join('/foo/bar', fi['filename']))
-    return file_instructions.file_instructions
+    return file_instructions
 
   def test_no_skip_no_take(self):
     instruction = tfrecords_reader._AbsoluteInstruction('train', None, None)
     files = self._get_files(instruction)
     self.assertEqual(files, [
-        {'skip': 0, 'take': -1, 'filename': self.PATH_PATTERN % i}
-        for i in range(5)])
+        shard_utils.FileInstruction(
+            filename=self.PATH_PATTERN % i, skip=0, take=-1, num_examples=n)
+        for i, n in enumerate([3, 2, 3, 2, 3])
+    ])
 
   def test_skip(self):
     # One file is not taken, one file is partially taken.
     instruction = tfrecords_reader._AbsoluteInstruction('train', 4, None)
     files = self._get_files(instruction)
     self.assertEqual(files, [
-        {'skip': 1, 'take': -1, 'filename': self.PATH_PATTERN % 1},
-        {'skip': 0, 'take': -1, 'filename': self.PATH_PATTERN % 2},
-        {'skip': 0, 'take': -1, 'filename': self.PATH_PATTERN % 3},
-        {'skip': 0, 'take': -1, 'filename': self.PATH_PATTERN % 4},
+        shard_utils.FileInstruction(
+            filename=self.PATH_PATTERN % 1, skip=1, take=-1, num_examples=1),
+        shard_utils.FileInstruction(
+            filename=self.PATH_PATTERN % 2, skip=0, take=-1, num_examples=3),
+        shard_utils.FileInstruction(
+            filename=self.PATH_PATTERN % 3, skip=0, take=-1, num_examples=2),
+        shard_utils.FileInstruction(
+            filename=self.PATH_PATTERN % 4, skip=0, take=-1, num_examples=3),
     ])
 
   def test_take(self):
@@ -85,9 +105,12 @@ class GetDatasetFilesTest(testing.TestCase):
     instruction = tfrecords_reader._AbsoluteInstruction('train', None, 6)
     files = self._get_files(instruction)
     self.assertEqual(files, [
-        {'skip': 0, 'take': -1, 'filename': self.PATH_PATTERN % 0},
-        {'skip': 0, 'take': -1, 'filename': self.PATH_PATTERN % 1},
-        {'skip': 0, 'take': 1, 'filename': self.PATH_PATTERN % 2},
+        shard_utils.FileInstruction(
+            filename=self.PATH_PATTERN % 0, skip=0, take=-1, num_examples=3),
+        shard_utils.FileInstruction(
+            filename=self.PATH_PATTERN % 1, skip=0, take=-1, num_examples=2),
+        shard_utils.FileInstruction(
+            filename=self.PATH_PATTERN % 2, skip=0, take=1, num_examples=1),
     ])
 
   def test_skip_take1(self):
@@ -95,7 +118,8 @@ class GetDatasetFilesTest(testing.TestCase):
     instruction = tfrecords_reader._AbsoluteInstruction('train', 1, 2)
     files = self._get_files(instruction)
     self.assertEqual(files, [
-        {'skip': 1, 'take': 1, 'filename': self.PATH_PATTERN % 0},
+        shard_utils.FileInstruction(
+            filename=self.PATH_PATTERN % 0, skip=1, take=1, num_examples=1),
     ])
 
   def test_skip_take2(self):
@@ -103,8 +127,10 @@ class GetDatasetFilesTest(testing.TestCase):
     instruction = tfrecords_reader._AbsoluteInstruction('train', 7, 9)
     files = self._get_files(instruction)
     self.assertEqual(files, [
-        {'skip': 2, 'take': -1, 'filename': self.PATH_PATTERN % 2},
-        {'skip': 0, 'take': 1, 'filename': self.PATH_PATTERN % 3},
+        shard_utils.FileInstruction(
+            filename=self.PATH_PATTERN % 2, skip=2, take=-1, num_examples=1),
+        shard_utils.FileInstruction(
+            filename=self.PATH_PATTERN % 3, skip=0, take=1, num_examples=1),
     ])
 
   def test_touching_boundaries(self):
@@ -141,6 +167,7 @@ class ReadInstructionTest(testing.TestCase):
         'train': 200,
         'test': 101,
         'validation': 30,
+        'dev-train': 10
     }
 
   def check_from_ri(self, ri, expected):
@@ -214,6 +241,9 @@ class ReadInstructionTest(testing.TestCase):
     # Slices resulting in empty datasets are valid with 'closest' rounding:
     self.check_from_spec('validation[:1%]', [('validation', None, 0)])
 
+    # Supports splits with '-' in name.
+    ri = self.check_from_spec('dev-train', [('dev-train', None, None)])
+
   def test_add(self):
     ri1 = tfrecords_reader.ReadInstruction.from_spec('train[10:20]')
     ri2 = tfrecords_reader.ReadInstruction.from_spec('test[10:20]')
@@ -273,11 +303,6 @@ class ReadInstructionTest(testing.TestCase):
 
 class ReaderTest(testing.TestCase):
 
-  SPLIT_INFOS = [
-      splits.SplitInfo(name='train', shard_lengths=[2, 3, 2, 3, 2]),  # 12 ex.
-      splits.SplitInfo(name='test', shard_lengths=[2, 3, 2]),  # 7 ex.
-  ]
-
   def setUp(self):
     super(ReaderTest, self).setUp()
     with absltest.mock.patch.object(example_parser,
@@ -298,22 +323,23 @@ class ReaderTest(testing.TestCase):
           num_examples, 0, [num_examples], path)
     serialized_records = [six.b(rec) for rec in records]
     for shard_spec in shard_specs:
-      tfrecords_writer._write_tfrecord_from_shard_spec(
+      _write_tfrecord_from_shard_spec(
           shard_spec, lambda unused_i: iter(serialized_records))
-
-  def _write_tfrecords(self):
-    self._write_tfrecord('train', 5, 'abcdefghijkl')
-    self._write_tfrecord('test', 3, 'mnopqrs')
+    return splits.SplitInfo(
+        name=split_name,
+        shard_lengths=[int(s.examples_number) for s in shard_specs],
+    )
 
   def test_nodata_instruction(self):
     # Given instruction corresponds to no data.
     with self.assertRaisesWithPredicateMatch(AssertionError,
                                              'corresponds to no data!'):
-      self.reader.read('mnist', 'train[0:0]', self.SPLIT_INFOS)
+      train_info = splits.SplitInfo(name='train', shard_lengths=[2, 3, 2, 3, 2])
+      self.reader.read('mnist', 'train[0:0]', [train_info])
 
   def test_noskip_notake(self):
-    self._write_tfrecord('train', 5, 'abcdefghijkl')
-    ds = self.reader.read('mnist', 'train', self.SPLIT_INFOS)
+    train_info = self._write_tfrecord('train', 5, 'abcdefghijkl')
+    ds = self.reader.read('mnist', 'train', [train_info])
     read_data = list(tfds.as_numpy(ds))
     self.assertEqual(read_data, [six.b(l) for l in 'abcdefghijkl'])
 
@@ -323,8 +349,8 @@ class ReaderTest(testing.TestCase):
           tf.data.experimental.cardinality(ds).numpy(), len(read_data))
 
   def test_overlap(self):
-    self._write_tfrecord('train', 5, 'abcdefghijkl')
-    ds = self.reader.read('mnist', 'train+train[:2]', self.SPLIT_INFOS)
+    train_info = self._write_tfrecord('train', 5, 'abcdefghijkl')
+    ds = self.reader.read('mnist', 'train+train[:2]', [train_info])
     read_data = list(tfds.as_numpy(ds))
     self.assertEqual(read_data, [six.b(l) for l in 'abcdefghijklab'])
 
@@ -334,9 +360,14 @@ class ReaderTest(testing.TestCase):
           tf.data.experimental.cardinality(ds).numpy(), len(read_data))
 
   def test_complex(self):
-    self._write_tfrecord('train', 5, 'abcdefghijkl')
-    self._write_tfrecord('test', 3, 'mnopqrs')
-    ds = self.reader.read('mnist', 'train[1:-1]+test[:-50%]', self.SPLIT_INFOS)
+    train_info = self._write_tfrecord('train', 5, 'abcdefghijkl')
+    test_info = self._write_tfrecord('test', 3, 'mnopqrs')
+    self.assertEqual(train_info.name, 'train')
+    self.assertEqual(test_info.name, 'test')
+    self.assertEqual(train_info.shard_lengths, [2, 3, 2, 3, 2])  # 12 ex.
+    self.assertEqual(test_info.shard_lengths, [2, 3, 2])  # 7 ex.
+    split_info = [train_info, test_info]
+    ds = self.reader.read('mnist', 'train[1:-1]+test[:-50%]', split_info)
     read_data = list(tfds.as_numpy(ds))
     self.assertEqual(read_data, [six.b(l) for l in 'bcdefghijkmno'])
 
@@ -346,8 +377,8 @@ class ReaderTest(testing.TestCase):
           tf.data.experimental.cardinality(ds).numpy(), len(read_data))
 
   def test_shuffle_files(self):
-    self._write_tfrecord('train', 5, 'abcdefghijkl')
-    ds = self.reader.read('mnist', 'train', self.SPLIT_INFOS,
+    train_info = self._write_tfrecord('train', 5, 'abcdefghijkl')
+    ds = self.reader.read('mnist', 'train', [train_info],
                           shuffle_files=True)
     shards = [  # The shards of the dataset:
         [b'a', b'b'],
@@ -368,13 +399,12 @@ class ReaderTest(testing.TestCase):
     self.assertGreater(len(set(read_data)), 10)
 
   def test_shuffle_deterministic(self):
-
-    self._write_tfrecord('train', 5, 'abcdefghijkl')
+    split_info = self._write_tfrecord('train', 5, 'abcdefghijkl')
     read_config = read_config_lib.ReadConfig(
         shuffle_seed=123,
     )
     ds = self.reader.read(
-        'mnist', 'train', self.SPLIT_INFOS,
+        'mnist', 'train', [split_info],
         read_config=read_config,
         shuffle_files=True)
     ds_values = list(tfds.as_numpy(ds))
@@ -385,16 +415,16 @@ class ReaderTest(testing.TestCase):
     ])
 
   def test_4fold(self):
-    self._write_tfrecord('train', 5, 'abcdefghijkl')
+    train_info = self._write_tfrecord('train', 5, 'abcdefghijkl')
     instructions = [
         tfrecords_reader.ReadInstruction('train', from_=k, to=k+25, unit='%')
         for k in range(0, 100, 25)]
-    tests = self.reader.read('mnist', instructions, self.SPLIT_INFOS)
+    tests = self.reader.read('mnist', instructions, [train_info])
     instructions = [
         (tfrecords_reader.ReadInstruction('train', to=k, unit='%') +
          tfrecords_reader.ReadInstruction('train', from_=k+25, unit='%'))
         for k in range(0, 100, 25)]
-    trains = self.reader.read('mnist', instructions, self.SPLIT_INFOS)
+    trains = self.reader.read('mnist', instructions, [train_info])
     read_tests = [list(r) for r in tfds.as_numpy(tests)]
     read_trains = [list(r) for r in tfds.as_numpy(trains)]
     self.assertEqual(read_tests, [[b'a', b'b', b'c'],
@@ -411,17 +441,55 @@ class ReaderTest(testing.TestCase):
     self._write_tfrecord('train', 4, 'abcdefghijkl')
     fname_pattern = 'mnist-train.tfrecord-0000%d-of-00004'
     ds = self.reader.read_files(
-        tfrecords_reader.FileInstructions(
-            file_instructions=[
-                {'filename': fname_pattern % 1, 'skip': 0, 'take': -1},
-                {'filename': fname_pattern % 3, 'skip': 1, 'take': 1},
-            ],
-            num_examples_per_shard=None,
-        ),
+        [
+            shard_utils.FileInstruction(
+                filename=fname_pattern % 1, skip=0, take=-1, num_examples=3),
+            shard_utils.FileInstruction(
+                filename=fname_pattern % 3, skip=1, take=1, num_examples=1),
+        ],
         read_config=read_config_lib.ReadConfig(),
-        shuffle_files=False)
+        shuffle_files=False,
+    )
     read_data = list(tfds.as_numpy(ds))
     self.assertEqual(read_data, [six.b(l) for l in 'defk'])
 
+  def test_input_context(self):
+    split_info = self._write_tfrecord('train', 5, 'abcdefghijkl')
+    self.assertEqual(split_info.shard_lengths, [2, 3, 2, 3, 2])
+
+    def read(num_workers, index):
+      return list(tfds.as_numpy(self.reader.read(
+          'mnist',
+          'train',
+          split_infos=[split_info],
+          read_config=read_config_lib.ReadConfig(
+              input_context=tf.distribute.InputContext(
+                  num_input_pipelines=num_workers,
+                  input_pipeline_id=index,
+              ),
+          ),
+          # Workers should read a deterministic subset of the examples, even
+          # if examples within one worker may be shuffled.
+          shuffle_files=True,
+      )))
+
+    def _b(bytes_str):
+      if six.PY2:
+        return list(bytes_str)
+      # Convert to List[bytes] (rather than List[int])
+      return [bytes([b]) for b in bytes_str]
+
+    # Read all the data (single pipeline)
+    self.assertCountEqual(read(num_workers=1, index=0), _b(b'abcdefghijkl'))
+    # Read part of the data (workers should not overlapp)
+    self.assertCountEqual(read(num_workers=3, index=0), _b(b'abhij'))  # 0, 3
+    self.assertCountEqual(read(num_workers=3, index=1), _b(b'cdekl'))  # 1, 4
+    self.assertEqual(read(num_workers=3, index=2), _b(b'fg'))  # Shards 2
+    # If num_workers == num_shards, then a single shard is read
+    self.assertEqual(read(num_workers=5, index=1), _b(b'cde'))  # Shard 1
+    # If num_workers > num_shards, raise error
+    with self.assertRaisesRegexp(ValueError, 'Cannot shard the pipeline'):
+      read(num_workers=6, index=0)
+
 if __name__ == '__main__':
   testing.test_main()
