@@ -26,15 +26,18 @@ import contextlib
 import hashlib
 import io
 import itertools
+import logging
 import os
 import random
 import string
 import sys
 import textwrap
-from typing import Iterator, TypeVar
+import threading
+from typing import Any, Callable, Iterator, List, TypeVar
 import uuid
 
 import six
+from six.moves import urllib
 import tensorflow.compat.v2 as tf
 from tensorflow_datasets.core import constants
 
@@ -59,15 +62,17 @@ else:
 memoize = functools.lru_cache
 
 
-T = TypeVar("T")
+T = TypeVar('T')
+
+Fn = TypeVar('Fn', bound=Callable[..., Any])
 
 
 def is_notebook():
   """Returns True if running in a notebook (Colab, Jupyter) environement."""
   # Inspired from the tfdm autonotebook code
   try:
-    from IPython import get_ipython    # pytype: disable=import-error  # pylint: disable=import-outside-toplevel,g-import-not-at-top
-    if "IPKernelApp" not in get_ipython().config:
+    import IPython  # pytype: disable=import-error  # pylint: disable=import-outside-toplevel,g-import-not-at-top
+    if 'IPKernelApp' not in IPython.get_ipython().config:
       return False  # Run in a IPython terminal
   except:  # pylint: disable=bare-except
     return False
@@ -78,10 +83,12 @@ def is_notebook():
 @contextlib.contextmanager
 def temporary_assignment(obj, attr, value):
   """Temporarily assign obj.attr to value."""
-  original = getattr(obj, attr, None)
+  original = getattr(obj, attr)
   setattr(obj, attr, value)
-  yield
-  setattr(obj, attr, original)
+  try:
+    yield
+  finally:
+    setattr(obj, attr, original)
 
 
 def zip_dict(*dicts):
@@ -91,6 +98,18 @@ def zip_dict(*dicts):
     yield key, tuple(d[key] for d in dicts)
 
 
+@contextlib.contextmanager
+def disable_logging():
+  """Temporarily disable the logging."""
+  logger = logging.getLogger()
+  logger_disabled = logger.disabled
+  logger.disabled = True
+  try:
+    yield
+  finally:
+    logger.disabled = logger_disabled
+
+
 class NonMutableDict(dict):
   """Dict where keys can only be added but not modified.
 
@@ -101,11 +120,11 @@ class NonMutableDict(dict):
 
   def __init__(self, *args, **kwargs):
     self._error_msg = kwargs.pop(
-        "error_msg",
-        "Try to overwrite existing key: {key}",
+        'error_msg',
+        'Try to overwrite existing key: {key}',
     )
     if kwargs:
-      raise ValueError("NonMutableDict cannot be initialized with kwargs.")
+      raise ValueError('NonMutableDict cannot be initialized with kwargs.')
     super(NonMutableDict, self).__init__(*args, **kwargs)
 
   def __setitem__(self, key, value):
@@ -134,8 +153,8 @@ class memoized_property(property):  # pylint: disable=invalid-name
     if obj is None:
       return self
     if self.fget is None:  # pytype: disable=attribute-error
-      raise AttributeError("unreadable attribute")
-    attr = "__cached_" + self.fget.__name__  # pytype: disable=attribute-error
+      raise AttributeError('unreadable attribute')
+    attr = '__cached_' + self.fget.__name__  # pytype: disable=attribute-error
     cached = getattr(obj, attr, None)
     if cached is None:
       cached = self.fget(obj)  # pytype: disable=attribute-error
@@ -170,7 +189,7 @@ def map_nested(function, data_struct, dict_only=False, map_tuple=False):
 def zip_nested(arg0, *args, **kwargs):
   """Zip data struct together and return a data struct with the same shape."""
   # Python 2 do not support kwargs only arguments
-  dict_only = kwargs.pop("dict_only", False)
+  dict_only = kwargs.pop('dict_only', False)
   assert not kwargs
 
   # Could add support for more exotic data_struct, like OrderedDict
@@ -192,7 +211,7 @@ def flatten_nest_dict(d):
   for k, v in d.items():
     if isinstance(v, dict):
       flat_dict.update({
-          "{}/{}".format(k, k2): v2 for k2, v2 in flatten_nest_dict(v).items()
+          '{}/{}'.format(k, k2): v2 for k2, v2 in flatten_nest_dict(v).items()
       })
     else:
       flat_dict[k] = v
@@ -211,7 +230,7 @@ def pack_as_nest_dict(flat_d, nest_d):
     if isinstance(v, dict):
       v_flat = flatten_nest_dict(v)
       sub_d = {
-          k2: flat_d.pop("{}/{}".format(k, k2)) for k2, _ in v_flat.items()
+          k2: flat_d.pop('{}/{}'.format(k, k2)) for k2, _ in v_flat.items()
       }
       # Recursivelly pack the dictionary
       nest_out_d[k] = pack_as_nest_dict(sub_d, v)
@@ -219,8 +238,8 @@ def pack_as_nest_dict(flat_d, nest_d):
       nest_out_d[k] = flat_d.pop(k)
   if flat_d:  # At the end, flat_d should be empty
     raise ValueError(
-        "Flat dict strucure do not match the nested dict. Extra keys: "
-        "{}".format(list(flat_d.keys())))
+        'Flat dict strucure do not match the nested dict. Extra keys: '
+        '{}'.format(list(flat_d.keys())))
   return nest_out_d
 
 
@@ -268,7 +287,7 @@ def as_proto_cls(proto_cls):
 
       def __init__(self, *args, **kwargs):
         super(ProtoCls, self).__setattr__(
-            "_ProtoCls__proto",
+            '_ProtoCls__proto',
             proto_cls(*args, **kwargs),
         )
 
@@ -292,11 +311,11 @@ def as_proto_cls(proto_cls):
         return self.__proto
 
       def __repr__(self):
-        return "<{cls_name}\n{proto_repr}\n>".format(
+        return '<{cls_name}\n{proto_repr}\n>'.format(
             cls_name=cls.__name__, proto_repr=repr(self.__proto))
 
     decorator_cls = type(cls.__name__, (cls, ProtoCls), {
-        "__doc__": cls.__doc__,
+        '__doc__': cls.__doc__,
     })
     return decorator_cls
   return decorator
@@ -304,9 +323,9 @@ def as_proto_cls(proto_cls):
 
 def _get_incomplete_path(filename):
   """Returns a temporary filename based on filename."""
-  random_suffix = "".join(
+  random_suffix = ''.join(
       random.choice(string.ascii_uppercase + string.digits) for _ in range(6))
-  return filename + ".incomplete" + random_suffix
+  return filename + '.incomplete' + random_suffix
 
 
 @contextlib.contextmanager
@@ -322,30 +341,28 @@ def incomplete_dir(dirname):
       tf.io.gfile.rmtree(tmp_dir)
 
 
-def tfds_dir():
-  """Path to tensorflow_datasets directory."""
+def tfds_dir() -> str:
+  """Path to tensorflow_datasets directory.
+
+  The difference with `tfds.core.get_tfds_path` is that this function can be
+  used for write access while `tfds.core.get_tfds_path` should be used for
+  read-only.
+
+  Returns:
+    tfds_dir: The root TFDS path.
+  """
   return os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
 
 
 @contextlib.contextmanager
 def atomic_write(path, mode):
   """Writes to path atomically, by writing to temp file and renaming it."""
-  tmp_path = "%s%s_%s" % (path, constants.INCOMPLETE_SUFFIX, uuid.uuid4().hex)
+  tmp_path = '%s%s_%s' % (path, constants.INCOMPLETE_SUFFIX, uuid.uuid4().hex)
   with tf.io.gfile.GFile(tmp_path, mode) as file_:
     yield file_
   tf.io.gfile.rename(tmp_path, path, overwrite=True)
 
 
-class abstractclassmethod(classmethod):  # pylint: disable=invalid-name
-  """Decorate a method to mark it as an abstract @classmethod."""
-
-  __isabstractmethod__ = True
-
-  def __init__(self, fn):
-    fn.__isabstractmethod__ = True
-    super(abstractclassmethod, self).__init__(fn)
-
-
 def get_tfds_path(relative_path):
   """Returns absolute path to file given path relative to tfds root."""
   path = os.path.join(tfds_dir(), relative_path)
@@ -356,7 +373,7 @@ def read_checksum_digest(path, checksum_cls=hashlib.sha256):
   """Given a hash constructor, returns checksum digest and size of file."""
   checksum = checksum_cls()
   size = 0
-  with tf.io.gfile.GFile(path, "rb") as f:
+  with tf.io.gfile.GFile(path, 'rb') as f:
     while True:
       block = f.read(io.DEFAULT_BUFFER_SIZE)
       size += len(block)
@@ -369,8 +386,8 @@ def read_checksum_digest(path, checksum_cls=hashlib.sha256):
 def reraise(prefix=None, suffix=None):
   """Reraise an exception with an additional message."""
   exc_type, exc_value, exc_traceback = sys.exc_info()
-  prefix = prefix or ""
-  suffix = "\n" + suffix if suffix else ""
+  prefix = prefix or ''
+  suffix = '\n' + suffix if suffix else ''
   msg = prefix + str(exc_value) + suffix
   six.reraise(exc_type, exc_type(msg), exc_traceback)
 
@@ -388,10 +405,10 @@ def rgetattr(obj, attr, *args):
   """Get attr that handles dots in attr name."""
   def _getattr(obj, attr):
     return getattr(obj, attr, *args)
-  return functools.reduce(_getattr, [obj] + attr.split("."))
+  return functools.reduce(_getattr, [obj] + attr.split('.'))
 
 
-def has_sufficient_disk_space(needed_bytes, directory="."):
+def has_sufficient_disk_space(needed_bytes, directory='.'):
   try:
     free_bytes = disk_usage(os.path.abspath(directory)).free
   except OSError:
@@ -404,14 +421,62 @@ def get_class_path(cls, use_tfds_prefix=True):
   if not isinstance(cls, type):
     cls = cls.__class__
   module_path = cls.__module__
-  if use_tfds_prefix and module_path.startswith("tensorflow_datasets"):
-    module_path = "tfds" + module_path[len("tensorflow_datasets"):]
-  return ".".join([module_path, cls.__name__])
+  if use_tfds_prefix and module_path.startswith('tensorflow_datasets'):
+    module_path = 'tfds' + module_path[len('tensorflow_datasets'):]
+  return '.'.join([module_path, cls.__name__])
 
 
 def get_class_url(cls):
   """Returns URL of given class or object."""
   cls_path = get_class_path(cls, use_tfds_prefix=False)
-  module_path, unused_class_name = cls_path.rsplit(".", 1)
-  module_path = module_path.replace(".", "/")
-  return constants.SRC_BASE_URL + module_path + ".py"
+  module_path, unused_class_name = cls_path.rsplit('.', 1)
+  module_path = module_path.replace('.', '/')
+  return constants.SRC_BASE_URL + module_path + '.py'
+
+
+def build_synchronize_decorator() -> Callable[[Fn], Fn]:
+  """Returns a decorator which prevents concurrent calls to functions.
+
+  Usage:
+    synchronized = build_synchronize_decorator()
+
+    @synchronized
+    def read_value():
+      ...
+
+    @synchronized
+    def write_value(x):
+      ...
+
+  Returns:
+    make_threadsafe (fct): The decorator which lock all functions to which it
+      is applied under a same lock
+  """
+  lock = threading.Lock()
+
+  def lock_decorator(fn: Fn) -> Fn:
+
+    @functools.wraps(fn)
+    def lock_decorated(*args, **kwargs):
+      with lock:
+        return fn(*args, **kwargs)
+
+    return lock_decorated
+
+  return lock_decorator
+
+
+def basename_from_url(url: str) -> str:
+  """Returns file name of file at given url."""
+  return os.path.basename(urllib.parse.urlparse(url).path) or 'unknown_name'
+
+
+def list_info_files(dir_path: str) -> List[str]:
+  """Returns name of info files within dir_path."""
+  # TODO(tfds): Is there a better filtering scheme which would be more
+  # resistant to future modifications (ex: tfrecord => other format)
+  return [
+      fname for fname in tf.io.gfile.listdir(dir_path)
+      if '.tfrecord' not in fname and
+      not tf.io.gfile.isdir(os.path.join(dir_path, fname))
+  ]
