@@ -42,5 +42,13 @@ class ShowExamplesTest(testing.TestCase):
   # TODO(tfds): Should add test when there isn't enough examples (ds.take(3))
 
 
+class ShowStatisticsTest(testing.TestCase):
+
+  def test_show_examples(self):
+    with testing.mock_data():
+      builder = registered.builder('imagenet2012')
+      visualization.show_statistics(builder.info)
+
+
 if __name__ == '__main__':
   testing.test_main()
