@@ -48,6 +48,11 @@ _DESCRIPTION = """\
 High-quality version of the CELEBA
 dataset, consisting of 30000 images in 1024 x 1024 resolution.
 
+Note: CelebAHQ dataset may contain potential bias. The fairness indicators
+[example](https://github.com/tensorflow/fairness-indicators/blob/master/fairness_indicators/documentation/examples/Fairness_Indicators_TFCO_CelebA_Case_Study.ipynb)
+goes into detail about several considerations to keep in mind while using the
+CelebAHQ dataset.
+
 WARNING: This dataset currently requires you to prepare images on your own.
 """
 
