@@ -44,9 +44,9 @@ from tensorflow_datasets.image_classification.flowers import TFFlowers
 from tensorflow_datasets.image_classification.food101 import Food101
 from tensorflow_datasets.image_classification.geirhos_conflict_stimuli import GeirhosConflictStimuli
 from tensorflow_datasets.image_classification.horses_or_humans import HorsesOrHumans
-from tensorflow_datasets.image_classification.image_folder import ImageLabelFolder
 from tensorflow_datasets.image_classification.imagenet import Imagenet2012
 from tensorflow_datasets.image_classification.imagenet2012_corrupted import Imagenet2012Corrupted
+from tensorflow_datasets.image_classification.imagenet2012_real import Imagenet2012Real
 from tensorflow_datasets.image_classification.imagenet2012_subset import Imagenet2012Subset
 from tensorflow_datasets.image_classification.imagenet_resized import ImagenetResized
 from tensorflow_datasets.image_classification.imagenette import Imagenette
