@@ -34,8 +34,8 @@ url = "http://laurencemoroney.com/rock-paper-scissors-dataset"
 }
 """
 
-_TRAIN_URL = "https://storage.googleapis.com/laurencemoroney-blog.appspot.com/rps.zip"
-_TEST_URL = "https://storage.googleapis.com/laurencemoroney-blog.appspot.com/rps-test-set.zip"
+_TRAIN_URL = "https://storage.googleapis.com/download.tensorflow.org/data/rps.zip"
+_TEST_URL = "https://storage.googleapis.com/download.tensorflow.org/data/rps-test-set.zip"
 
 _IMAGE_SIZE = 300
 _IMAGE_SHAPE = (_IMAGE_SIZE, _IMAGE_SIZE, 3)
