@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# Lint as: python3
 """The Waymo Open Dataset. See waymo.com/open."""
 
 from __future__ import absolute_import
@@ -27,10 +28,12 @@ from tensorflow_datasets.proto import waymo_dataset_pb2 as open_dataset
 import tensorflow_datasets.public_api as tfds
 
 _CITATION = """
-@misc{waymo_open_dataset,
-  title = {Waymo Open Dataset: An autonomous driving dataset},
-  website = {url{https://www.waymo.com/open}},
-  year = {2020}
+@InProceedings{Sun_2020_CVPR,
+author = {Sun, Pei and Kretzschmar, Henrik and Dotiwalla, Xerxes and Chouard, Aurelien and Patnaik, Vijaysai and Tsui, Paul and Guo, James and Zhou, Yin and Chai, Yuning and Caine, Benjamin and Vasudevan, Vijay and Han, Wei and Ngiam, Jiquan and Zhao, Hang and Timofeev, Aleksei and Ettinger, Scott and Krivokon, Maxim and Gao, Amy and Joshi, Aditya and Zhang, Yu and Shlens, Jonathon and Chen, Zhifeng and Anguelov, Dragomir},
+title = {Scalability in Perception for Autonomous Driving: Waymo Open Dataset},
+booktitle = {The IEEE/CVF Conference on Computer Vision and Pattern Recognition (CVPR)},
+month = {June},
+year = {2020}
 }
 """
 
@@ -62,7 +65,6 @@ class WaymoOpenDataset(tfds.core.BeamBasedBuilder):
   VERSION = tfds.core.Version("0.1.0")
   _CLOUD_BUCKET = "gs://waymo_open_dataset_v_1_0_0_individual_files/"
 
-
   def _info(self):
 
     # Annotation descriptions are in the object development kit.
@@ -169,7 +171,7 @@ def _generate_images_and_annotations(tf_record_file):
   dataset = tf.data.TFRecordDataset(tf_record_file, compression_type="")
   for data in dataset:
     frame = open_dataset.Frame()
-    frame.ParseFromString(bytearray(data.numpy()))
+    frame.ParseFromString(bytearray(data.numpy()))  # pytype: disable=wrong-arg-types
 
     image_and_annotation = {
         "context": {
