@@ -16,10 +16,13 @@
 # Lint as: python3
 """Text datasets."""
 
+from tensorflow_datasets.text.anli import Anli
 from tensorflow_datasets.text.blimp import Blimp
 from tensorflow_datasets.text.c4 import C4
 from tensorflow_datasets.text.cfq import CFQ
+from tensorflow_datasets.text.cfq import CFQConfig
 from tensorflow_datasets.text.civil_comments import CivilComments
+from tensorflow_datasets.text.clinc_oos import ClincOOS
 from tensorflow_datasets.text.cos_e import CosE
 from tensorflow_datasets.text.definite_pronoun_resolution import DefinitePronounResolution
 from tensorflow_datasets.text.eraser_multi_rc import EraserMultiRc
@@ -28,6 +31,7 @@ from tensorflow_datasets.text.gap import Gap
 from tensorflow_datasets.text.glue import Glue
 from tensorflow_datasets.text.imdb import IMDBReviews
 from tensorflow_datasets.text.imdb import IMDBReviewsConfig
+from tensorflow_datasets.text.irc_disentanglement import IrcDisentanglement
 from tensorflow_datasets.text.librispeech_lm import LibrispeechLm
 from tensorflow_datasets.text.lm1b import Lm1b
 from tensorflow_datasets.text.lm1b import Lm1bConfig
@@ -35,19 +39,20 @@ from tensorflow_datasets.text.math_dataset import MathDataset
 from tensorflow_datasets.text.movie_rationales import MovieRationales
 from tensorflow_datasets.text.multi_nli import MultiNLI
 from tensorflow_datasets.text.multi_nli_mismatch import MultiNLIMismatch
-from tensorflow_datasets.text.natural_questions import NaturalQuestions
+from tensorflow_datasets.text.openbookqa import Openbookqa
+from tensorflow_datasets.text.pg19 import Pg19
 from tensorflow_datasets.text.qa4mre import Qa4mre
+from tensorflow_datasets.text.reddit_disentanglement import RedditDisentanglement
 from tensorflow_datasets.text.scan import Scan
 from tensorflow_datasets.text.scan import ScanConfig
 from tensorflow_datasets.text.scicite import Scicite
 from tensorflow_datasets.text.snli import Snli
-from tensorflow_datasets.text.squad import Squad
 from tensorflow_datasets.text.super_glue import SuperGlue
 from tensorflow_datasets.text.tiny_shakespeare import TinyShakespeare
-from tensorflow_datasets.text.trivia_qa import TriviaQA
-from tensorflow_datasets.text.web_questions import WebQuestions
 from tensorflow_datasets.text.wiki40b import Wiki40b
 from tensorflow_datasets.text.wikipedia import Wikipedia
+from tensorflow_datasets.text.wikipedia_toxicity_subtypes import WikipediaToxicitySubtypes
+from tensorflow_datasets.text.winogrande import Winogrande
+from tensorflow_datasets.text.wordnet import Wordnet
 from tensorflow_datasets.text.xnli import Xnli
 from tensorflow_datasets.text.yelp_polarity import YelpPolarityReviews
-
