@@ -14,7 +14,7 @@
 # limitations under the License.
 
 # Lint as: python3
-"""TODO(scicite): Add a description here."""
+"""Test for scicite dataset."""
 
 from __future__ import absolute_import
 from __future__ import division
